@@ -25,11 +25,20 @@ __all__ = [
 
 import json
 import os
+import shutil
+from textwrap import dedent
 
-from gi.repository import Click
+from gi.repository import Click, GLib
 
 from click.json_helpers import json_array_to_python, json_object_to_python
-from click.tests.helpers import TestCase, mkfile
+from click.tests.gimock_types import Passwd
+from click.tests.helpers import (
+    TestCase,
+    make_installed_click,
+    mkfile,
+    make_file_with_content,
+    touch,
+)
 
 
 class TestClickUser(TestCase):
@@ -51,7 +60,8 @@ class TestClickUser(TestCase):
         a_1_0 = os.path.join(self.temp_dir, "custom", "a", "1.0")
         os.makedirs(a_1_0)
         with mkfile(os.path.join(a_1_0, ".click", "info", "a.manifest")) as m:
-            json.dump({"name": "a", "version": "1.0"}, m)
+            json.dump({"name": "a", "version": "1.0",
+                       "hooks": {"a-app": {}}}, m)
         b_2_0 = os.path.join(self.temp_dir, "custom", "b", "2.0")
         os.makedirs(b_2_0)
         with mkfile(os.path.join(b_2_0, ".click", "info", "b.manifest")) as m:
@@ -90,11 +100,106 @@ class TestClickUser(TestCase):
                 os.path.join(db_root, ".click", "users", "test-user"),
                 registry.get_overlay_db())
 
+    def test_new_db_not_directory(self):
+        with self.run_in_subprocess(
+                "click_get_db_dir", "g_get_user_name") as (enter, preloads):
+            enter()
+            path = os.path.join(self.temp_dir, "file")
+            touch(path)
+            preloads["click_get_db_dir"].side_effect = (
+                lambda: self.make_string(path))
+            self.assertRaisesFileError(
+                GLib.FileError.NOTDIR, Click.User.for_user, None, None)
+
     def test_get_overlay_db(self):
         self.assertEqual(
             os.path.join(self.temp_dir, ".click", "users", "user"),
             Click.User.for_user(self.db, "user").get_overlay_db())
 
+    def test_ensure_db_ownership(self):
+        # getpwnam results are cached properly, in a way that doesn't fail
+        # due to confusion with getpwnam returning a pointer to a static
+        # buffer.
+        with self.run_in_subprocess(
+                "chown", "geteuid", "getpwnam") as (enter, preloads):
+            enter()
+            preloads["geteuid"].return_value = 0
+            getpwnam_result = Passwd()
+
+            def getpwnam_side_effect(name):
+                if name == b"clickpkg":
+                    getpwnam_result.pw_uid = 1
+                    getpwnam_result.pw_gid = 1
+                else:
+                    getpwnam_result.pw_uid = 2
+                    getpwnam_result.pw_gid = 2
+                return self.make_pointer(getpwnam_result)
+
+            preloads["getpwnam"].side_effect = getpwnam_side_effect
+            registry = Click.User.for_user(self.db, "user")
+            os.makedirs(os.path.join(self.temp_dir, "a", "1.0"))
+            click_dir = os.path.join(self.temp_dir, ".click")
+
+            registry.set_version("a", "1.0")
+            self.assertEqual(3, preloads["chown"].call_count)
+            preloads["chown"].assert_any_call(click_dir.encode(), 1, 1)
+            preloads["chown"].assert_any_call(
+                os.path.join(click_dir, "users").encode(), 1, 1)
+            preloads["chown"].assert_any_call(
+                os.path.join(click_dir, "users", "user").encode(), 2, 2)
+
+            # Try again, now that both password file entries should be
+            # cached.
+            shutil.rmtree(os.path.join(self.temp_dir, ".click"))
+            preloads["chown"].reset_mock()
+            registry.set_version("a", "1.0")
+            self.assertEqual(3, preloads["chown"].call_count)
+            preloads["chown"].assert_any_call(click_dir.encode(), 1, 1)
+            preloads["chown"].assert_any_call(
+                os.path.join(click_dir, "users").encode(), 1, 1)
+            preloads["chown"].assert_any_call(
+                os.path.join(click_dir, "users", "user").encode(), 2, 2)
+
+    def test_ensure_db_mkdir_fails(self):
+        with self.run_in_subprocess("mkdir") as (enter, preloads):
+            enter()
+            preloads["mkdir"].return_value = -1
+            registry = Click.User.for_user(self.db, "user")
+            self.assertRaisesUserError(
+                Click.UserError.CREATE_DB, registry.set_version, "a", "1.0")
+
+    def test_ensure_db_chown_fails(self):
+        with self.run_in_subprocess(
+                "chown", "geteuid", "getpwnam") as (enter, preloads):
+            enter()
+            preloads["geteuid"].return_value = 0
+            getpwnam_result = Passwd()
+
+            def getpwnam_side_effect(name):
+                if name == b"clickpkg":
+                    getpwnam_result.pw_uid = 1
+                    getpwnam_result.pw_gid = 1
+                else:
+                    getpwnam_result.pw_uid = 2
+                    getpwnam_result.pw_gid = 2
+                return self.make_pointer(getpwnam_result)
+
+            preloads["getpwnam"].side_effect = getpwnam_side_effect
+            preloads["chown"].return_value = -1
+            registry = Click.User.for_user(self.db, "user")
+            self.assertRaisesUserError(
+                Click.UserError.CHOWN_DB, registry.set_version, "a", "1.0")
+
+    def test_ensure_db_getpwnam_fails(self):
+        with self.run_in_subprocess(
+                "geteuid", "getpwnam") as (enter, preloads):
+            enter()
+            preloads["geteuid"].return_value = 0
+            preloads["getpwnam"].return_value = None
+            registry = Click.User.for_user(self.db, "user")
+            self.assertRaisesUserError(
+                Click.UserError.GETPWNAM, registry.set_version, "a", "1.0")
+
     def test_get_package_names_missing(self):
         db = Click.DB()
         db.add(os.path.join(self.temp_dir, "nonexistent"))
@@ -452,3 +557,35 @@ class TestClickUser(TestCase):
         self.assertEqual("2.0", registry.get_version("b"))
         self.assertEqual(b_overlay, registry.get_path("b"))
         self.assertTrue(registry.is_removable("b"))
+
+
+class StopAppTestCase(TestCase):
+
+    def setUp(self):
+        super(StopAppTestCase, self).setUp()
+        self.use_temp_dir()
+        self.db = Click.DB()
+        self.db.add(self.temp_dir)
+
+        # setup fake app_stop
+        fake_app_stop = os.path.join(self.temp_dir, "bin", "ubuntu-app-stop")
+        self.fake_app_stop_output = os.path.join(
+            self.temp_dir, "fake-app-stop.out")
+        fake_app_stop_content = dedent("""\
+        #!/bin/sh
+        echo "$@" >> %s
+        """ % self.fake_app_stop_output)
+        make_file_with_content(fake_app_stop, fake_app_stop_content, 0o755)
+        # its ok to modify env here, click.helpers.TestCase will take care
+        # of it
+        os.environ["PATH"] = "%s:%s" % (
+            os.path.dirname(fake_app_stop), os.environ["PATH"])
+
+    def test_app_stops_on_remove(self):
+        make_installed_click(self.db, self.temp_dir, "meep", "2.0",
+                             {"hooks": {"a-app": {}}})
+        registry = Click.User.for_user(self.db, "user")
+        registry.remove("meep")
+        # ensure that stop was called with the right app
+        with open(self.fake_app_stop_output) as f:
+            self.assertEqual("meep_a-app_2.0", f.read().strip())
